#ifndef _ARCH_ARM_SAM7X_H_
#define _ARCH_ARM_SAM7X_H_
/*
 * Copyright (C) 2005 by egnite Software GmbH. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */

/*!
 * Change to sam7x.h
 *     By HU Daoxu, 2006/05/26
 *
 * \file arch/arm/at91.h
 * \brief AT91 peripherals.
 *
 * \verbatim
 *
 * $Log: at91.h,v $
 * Revision 1.3  2006/01/05 16:52:49  haraldkipp
 * Baudrate calculation is now based on NutGetCpuClock().
 * The AT91_US_BAUD macro had been marked deprecated.
 *
 * Revision 1.2  2005/11/20 14:44:14  haraldkipp
 * Register offsets added.
 *
 * Revision 1.1  2005/10/24 10:31:13  haraldkipp
 * Moved from parent directory.
 *
 *
 * \endverbatim
 */

#define FLASH_BASE              0x100000UL
#define RAM_BASE                0x200000UL

/*!
 * \addtogroup xgNutArchArmAt91Tc
 */

/*@{*/
#define TC_BASE                 0xFFFA0000      /*!< \brief TC base address. */

/*! \name Timer Counter Control Register */
/*@{*/
#define TC0_CCR         (TC_BASE + 0x00)        /*!< \brief Channel 0 control register address. */
#define TC1_CCR         (TC_BASE + 0x40)        /*!< \brief Channel 1 control register address. */
#define TC2_CCR         (TC_BASE + 0x80)        /*!< \brief Channel 2 control register address. */
#define TC_CLKEN                0x00000001      /*!< \brief Clock enable command. */
#define TC_CLKDIS               0x00000002      /*!< \brief Clock disable command. */
#define TC_SWTRG                0x00000004      /*!< \brief Software trigger command. */
/*@}*/

/*! \name Timer Counter Channel Mode Register */
/*@{*/
#define TC0_CMR         (TC_BASE + 0x04)        /*!< \brief Channel 0 mode register address. */
#define TC1_CMR         (TC_BASE + 0x44)        /*!< \brief Channel 1 mode register address. */
#define TC2_CMR         (TC_BASE + 0x84)        /*!< \brief Channel 2 mode register address. */

#define TC_CLKS                 0x00000007      /*!< \brief Clock selection mask. */
#define TC_CLKS_MCK2            0x00000000      /*!< \brief Selects MCK / 2. */
#define TC_CLKS_MCK8            0x00000001      /*!< \brief Selects MCK / 8. */
#define TC_CLKS_MCK32           0x00000002      /*!< \brief Selects MCK / 32. */
#define TC_CLKS_MCK128          0x00000003      /*!< \brief Selects MCK / 128. */
#define TC_CLKS_MCK1024         0x00000004      /*!< \brief Selects MCK / 1024. */
#define TC_CLKS_XC0             0x00000005      /*!< \brief Selects external clock 0. */
#define TC_CLKS_XC1             0x00000006      /*!< \brief Selects external clock 1. */
#define TC_CLKS_XC2             0x00000007      /*!< \brief Selects external clock 2. */

#define TC_CLKI                 0x00000008      /*!< \brief Increments on falling edge. */

#define TC_BURST                0x00000030      /*!< \brief Burst signal selection mask. */
#define TC_BURST_NONE           0x00000000      /*!< \brief Clock is not gated by an external signal. */
#define TC_BUSRT_XC0            0x00000010      /*!< \brief ANDed with external clock 0. */
#define TC_BURST_XC1            0x00000020      /*!< \brief ANDed with external clock 1. */
#define TC_BURST_XC2            0x00000030      /*!< \brief ANDed with external clock 2. */

#define TC_CPCTRG               0x00004000      /*!< \brief RC Compare Enable Trigger Enable. */

#define TC_WAVE                 0x00008000      /*!< \brief Selects waveform mode. */
#define TC_CAPT                 0x00000000      /*!< \brief Selects capture mode. */
/*@}*/

/*! \name Capture Mode */
/*@{*/
#define TC_LDBSTOP              0x00000040      /*!< \brief Counter clock stopped on RB loading. */
#define TC_LDBDIS               0x00000080      /*!< \brief Counter clock disabled on RB loading. */

#define TC_ETRGEDG              0x00000300      /*!< \brief External trigger edge selection mask. */
#define TC_ETRGEDG_RISING_EDGE  0x00000100      /*!< \brief Trigger on external rising edge. */
#define TC_ETRGEDG_FALLING_EDGE 0x00000200      /*!< \brief Trigger on external falling edge. */
#define TC_ETRGEDG_BOTH_EDGE    0x00000300      /*!< \brief Trigger on both external edges. */

#define TC_ABETRG               0x00000400      /*!< \brief TIOA or TIOB external trigger selection mask. */
#define TC_ABETRG_TIOB          0x00000000      /*!< \brief TIOB used as an external trigger. */
#define TC_ABETRG_TIOA          0x00000400      /*!< \brief TIOA used as an external trigger. */

#define TC_LDRA                 0x00030000      /*!< \brief RA loading selection mask. */
#define TC_LDRA_RISING_EDGE     0x00010000      /*!< \brief Load RA on rising edge of TIOA. */
#define TC_LDRA_FALLING_EDGE    0x00020000      /*!< \brief Load RA on falling edge of TIOA. */
#define TC_LDRA_BOTH_EDGE       0x00030000      /*!< \brief Load RA on any edge of TIOA. */

#define TC_LDRB                 0x000C0000      /*!< \brief RB loading selection mask. */
#define TC_LDRB_RISING_EDGE     0x00040000      /*!< \brief Load RB on rising edge of TIOA. */
#define TC_LDRB_FALLING_EDGE    0x00080000      /*!< \brief Load RB on falling edge of TIOA. */
#define TC_LDRB_BOTH_EDGE       0x000C0000      /*!< \brief Load RB on any edge of TIOA. */

/*@}*/

/*! \name Waveform Mode */
/*@{*/
#define TC_CPCSTOP              0x00000040      /*!< \brief Counter clock stopped on RC compare. */
#define TC_CPCDIS               0x00000080      /*!< \brief Counter clock disabled on RC compare. */

#define TC_EEVTEDG              0x00000300      /*!< \brief External event edge selection mask. */
#define TC_EEVTEDG_RISING_EDGE  0x00000100      /*!< \brief External event on rising edge.. */
#define TC_EEVTEDG_FALLING_EDGE 0x00000200      /*!< \brief External event on falling edge.. */
#define TC_EEVTEDG_BOTH_EDGE    0x00000300      /*!< \brief External event on any edge.. */

#define TC_EEVT                 0x00000C00      /*!< \brief External event selection mask. */
#define TC_EEVT_TIOB            0x00000000      /*!< \brief TIOB selected as external event. */
#define TC_EEVT_XC0             0x00000400      /*!< \brief XC0 selected as external event. */
#define TC_EEVT_XC1             0x00000800      /*!< \brief XC1 selected as external event. */
#define TC_EEVT_XC2             0x00000C00      /*!< \brief XC2 selected as external event. */

#define TC_ENETRG               0x00001000      /*!< \brief External event trigger enable. */

#define TC_ACPA                 0x00030000      /*!< \brief Masks RA compare effect on TIOA. */
#define TC_ACPA_SET_OUTPUT      0x00010000      /*!< \brief RA compare sets TIOA. */
#define TC_ACPA_CLEAR_OUTPUT    0x00020000      /*!< \brief RA compare clears TIOA. */
#define TC_ACPA_TOGGLE_OUTPUT   0x00030000      /*!< \brief RA compare toggles TIOA. */

#define TC_ACPC                 0x000C0000      /*!< \brief Masks RC compare effect on TIOA. */
#define TC_ACPC_SET_OUTPUT      0x00040000      /*!< \brief RC compare sets TIOA. */
#define TC_ACPC_CLEAR_OUTPUT    0x00080000      /*!< \brief RC compare clears TIOA. */
#define TC_ACPC_TOGGLE_OUTPUT   0x000C0000      /*!< \brief RC compare toggles TIOA. */

#define TC_AEEVT                0x00300000      /*!< \brief Masks external event effect on TIOA. */
#define TC_AEEVT_SET_OUTPUT     0x00100000      /*!< \brief External event sets TIOA. */
#define TC_AEEVT_CLEAR_OUTPUT   0x00200000      /*!< \brief External event clears TIOA. */
#define TC_AEEVT_TOGGLE_OUTPUT  0x00300000      /*!< \brief External event toggles TIOA. */

#define TC_ASWTRG               0x00C00000      /*!< \brief Masks software trigger effect on TIOA. */
#define TC_ASWTRG_SET_OUTPUT    0x00400000      /*!< \brief Software trigger sets TIOA. */
#define TC_ASWTRG_CLEAR_OUTPUT  0x00800000      /*!< \brief Software trigger clears TIOA. */
#define TC_ASWTRG_TOGGLE_OUTPUT 0x00C00000      /*!< \brief Software trigger toggles TIOA. */

#define TC_BCPB                 0x03000000      /*!< \brief Masks RB compare effect on TIOB. */
#define TC_BCPB_SET_OUTPUT      0x01000000      /*!< \brief RB compare sets TIOB. */
#define TC_BCPB_CLEAR_OUTPUT    0x02000000      /*!< \brief RB compare clears TIOB. */
#define TC_BCPB_TOGGLE_OUTPUT   0x03000000      /*!< \brief RB compare toggles TIOB. */

#define TC_BCPC                 0x0C000000      /*!< \brief Masks RC compare effect on TIOB. */
#define TC_BCPC_SET_OUTPUT      0x04000000      /*!< \brief RC compare sets TIOB. */
#define TC_BCPC_CLEAR_OUTPUT    0x08000000      /*!< \brief RC compare clears TIOB. */
#define TC_BCPC_TOGGLE_OUTPUT   0x0C000000      /*!< \brief RC compare toggles TIOB. */

#define TC_BEEVT                0x30000000      /*!< \brief Masks external event effect on TIOB. */
#define TC_BEEVT_SET_OUTPUT     0x10000000      /*!< \brief External event sets TIOB. */
#define TC_BEEVT_CLEAR_OUTPUT   0x20000000      /*!< \brief External event clears TIOB. */
#define TC_BEEVT_TOGGLE_OUTPUT  0x30000000      /*!< \brief External event toggles TIOB. */

#define TC_BSWTRG               0xC0000000      /*!< \brief Masks software trigger effect on TIOB. */
#define TC_BSWTRG_SET_OUTPUT    0x40000000      /*!< \brief Software trigger sets TIOB. */
#define TC_BSWTRG_CLEAR_OUTPUT  0x80000000      /*!< \brief Software trigger clears TIOB. */
#define TC_BSWTRG_TOGGLE_OUTPUT 0xC0000000      /*!< \brief Software trigger toggles TIOB. */
/*@}*/

/*! \name Counter Value Register */
/*@{*/
#define TC0_CV          (TC_BASE + 0x10)        /*!< \brief Counter 0 value. */
#define TC1_CV          (TC_BASE + 0x50)        /*!< \brief Counter 1 value. */
#define TC2_CV          (TC_BASE + 0x90)        /*!< \brief Counter 2 value. */
/*@}*/

/*! \name Timer Counter Register A */
/*@{*/
#define TC0_RA          (TC_BASE + 0x14)        /*!< \brief Channel 0 register A. */
#define TC1_RA          (TC_BASE + 0x54)        /*!< \brief Channel 1 register A. */
#define TC2_RA          (TC_BASE + 0x94)        /*!< \brief Channel 2 register A. */
/*@}*/

/*! \name Timer Counter Register B */
/*@{*/
#define TC0_RB          (TC_BASE + 0x18)        /*!< \brief Channel 0 register B. */
#define TC1_RB          (TC_BASE + 0x58)        /*!< \brief Channel 1 register B. */
#define TC2_RB          (TC_BASE + 0x98)        /*!< \brief Channel 2 register B. */
/*@}*/

/*! \name Timer Counter Register C */
/*@{*/
#define TC0_RC          (TC_BASE + 0x1C)        /*!< \brief Channel 0 register C. */
#define TC1_RC          (TC_BASE + 0x5C)        /*!< \brief Channel 1 register C. */
#define TC2_RC          (TC_BASE + 0x9C)        /*!< \brief Channel 2 register C. */
/*@}*/


/*! \name Timer Counter Status and Interrupt Registers */
/*@{*/
#define TC0_SR          (TC_BASE + 0x20)        /*!< \brief Status register address. */
#define TC1_SR          (TC_BASE + 0x60)        /*!< \brief Status register address. */
#define TC2_SR          (TC_BASE + 0xA0)        /*!< \brief Status register address. */

#define TC0_IER         (TC_BASE + 0x24)        /*!< \brief Channel 0 interrupt enable register address. */
#define TC1_IER         (TC_BASE + 0x64)        /*!< \brief Channel 1 interrupt enable register address. */
#define TC2_IER         (TC_BASE + 0xA4)        /*!< \brief Channel 2 interrupt enable register address. */

#define TC0_IDR         (TC_BASE + 0x28)        /*!< \brief Channel 0 interrupt disable register address. */
#define TC1_IDR         (TC_BASE + 0x68)        /*!< \brief Channel 1 interrupt disable register address. */
#define TC2_IDR         (TC_BASE + 0xA8)        /*!< \brief Channel 2 interrupt disable register address. */

#define TC0_IMR         (TC_BASE + 0x2C)        /*!< \brief Channel 0 interrupt mask register address. */
#define TC1_IMR         (TC_BASE + 0x6C)        /*!< \brief Channel 1 interrupt mask register address. */
#define TC2_IMR         (TC_BASE + 0xAC)        /*!< \brief Channel 2 interrupt mask register address. */

#define TC_COVFS                0x00000001      /*!< \brief Counter overflow flag. */
#define TC_LOVRS                0x00000002      /*!< \brief Load overrun flag. */
#define TC_CPAS                 0x00000004      /*!< \brief RA compare flag. */
#define TC_CPBS                 0x00000008      /*!< \brief RB compare flag. */
#define TC_CPCS                 0x00000010      /*!< \brief RC compare flag. */
#define TC_LDRAS                0x00000020      /*!< \brief RA loading flag. */
#define TC_LDRBS                0x00000040      /*!< \brief RB loading flag. */
#define TC_ETRGS                0x00000080      /*!< \brief External trigger flag. */
#define TC_CLKSTA               0x00010000      /*!< \brief Clock enable flag. */
#define TC_MTIOA                0x00020000      /*!< \brief TIOA flag. */
#define TC_MTIOB                0x00040000      /*!< \brief TIOB flag. */
/*@}*/

/*! \name Timer Counter Block Control Register */
/*@{*/
#define TC_BCR          (TC_BASE + 0xC0)        /*!< \brief Block control register address. */
#define TC_SYNC                 0x00000001      /*!< \brief Synchronisation trigger */
/*@}*/

/*! \name Timer Counter Block Mode Register */
/*@{*/
#define TC_BMR          (TC_BASE + 0xC4)        /*!< \brief Block mode register address. */
#define TC_TC0XC0S              0x00000003      /*!< \brief External clock signal 0 selection mask. */
#define TC_TCLK0XC0             0x00000000      /*!< \brief Selects TCLK0. */
#define TC_NONEXC0              0x00000001      /*!< \brief None selected. */
#define TC_TIOA1XC0             0x00000002      /*!< \brief Selects TIOA1. */
#define TC_TIOA2XC0             0x00000003      /*!< \brief Selects TIOA2. */

#define TC_TC1XC1S              0x0000000C      /*!< \brief External clock signal 1 selection mask. */
#define TC_TCLK1XC1             0x00000000      /*!< \brief Selects TCLK1. */
#define TC_NONEXC1              0x00000004      /*!< \brief None selected. */
#define TC_TIOA0XC1             0x00000008      /*!< \brief Selects TIOA0. */
#define TC_TIOA2XC1             0x0000000C      /*!< \brief Selects TIOA2. */

#define TC_TC2XC2S              0x00000030      /*!< \brief External clock signal 2 selection mask. */
#define TC_TCLK2XC2             0x00000000      /*!< \brief Selects TCLK2. */
#define TC_NONEXC2              0x00000010      /*!< \brief None selected. */
#define TC_TIOA0XC2             0x00000020      /*!< \brief Selects TIOA0. */
#define TC_TIOA1XC2             0x00000030      /*!< \brief Selects TIOA1. */
/*@}*/

/*@} xgNutArchArmAt91Tc */

/*!
 * \addtogroup xgNutArchArmAt91Us
 */
/*@{*/

#define USART1_BASE             0xFFFC4000      /*!< \brief USART 1 base address. */
#define USART0_BASE             0xFFFC0000      /*!< \brief USART 0 base address. */
#define DBGU_BASE               0xFFFFF200      /*!< \brief DBGU base address. */

/*! \name USART Control Register */
/*@{*/
#define US_CR_OFF               0x00000000      /*!< \brief USART control register offset. */
#define US0_CR  (USART0_BASE + US_CR_OFF)       /*!< \brief Channel 0 control register address. */
#define US1_CR  (USART1_BASE + US_CR_OFF)       /*!< \brief Channel 1 control register address. */
#define DBGU_CR (DBGU_BASE + US_CR_OFF)         /*!< \brief DBGU control register address. */
#define US_RSTRX                0x00000004      /*!< \brief Reset receiver */
#define US_RSTTX                0x00000008      /*!< \brief Reset transmitter */
#define US_RXEN                 0x00000010      /*!< \brief Receiver enable */
#define US_RXDIS                0x00000020      /*!< \brief Receiver disable */
#define US_TXEN                 0x00000040      /*!< \brief Transmitter enable */
#define US_TXDIS                0x00000080      /*!< \brief Transmitter disable */
#define US_RSTSTA               0x00000100      /*!< \brief Reset status bits */
#define US_STTBRK               0x00000200      /*!< \brief Start break */
#define US_STPBRK               0x00000400      /*!< \brief Stop break */
#define US_STTTO                0x00000800      /*!< \brief Start timeout */
#define US_SENDA                0x00001000      /*!< \brief Send next byte with address bit set. */
/*@}*/

/*! \name Mode Register */
/*@{*/
#define US_MR_OFF               0x00000004      /*!< \brief USART mode register offset. */
#define US0_MR  (USART0_BASE + US_MR_OFF)       /*!< \brief Channel 0 mode register address. */
#define US1_MR  (USART1_BASE + US_MR_OFF)       /*!< \brief Channel 1 mode register address. */
#define DBGU_MR (DBGU_BASE + US_MR_OFF)         /*!< \brief DBGU mode register address. */

#define US_CLKS                 0x00000030      /*!< \brief Clock selection mask. */
#define US_CLKS_MCK             0x00000000      /*!< \brief Master clock. */
#define US_CLKS_MCK8            0x00000010      /*!< \brief Master clock divided by 8. */
#define US_CLKS_SCK             0x00000020      /*!< \brief External clock. */
#define US_CLKS_SLCK            0x00000030      /*!< \brief Slow clock. */

#define US_CHRL                 0x000000C0      /*!< \brief Masks data length. */
#define US_CHRL_5               0x00000000      /*!< \brief 5 data bits. */
#define US_CHRL_6               0x00000040      /*!< \brief 6 data bits. */
#define US_CHRL_7               0x00000080      /*!< \brief 7 data bits. */
#define US_CHRL_8               0x000000C0      /*!< \brief 8 data bits. */

#define US_SYNC                 0x00000100      /*!< \brief Synchronous mode enable. */

#define US_PAR                  0x00000E00      /*!< \brief Parity mode mask. */
#define US_PAR_EVEN             0x00000000      /*!< \brief Even parity */
#define US_PAR_ODD              0x00000200      /*!< \brief Odd parity */
#define US_PAR_SPACE            0x00000400      /*!< \brief Space parity. */
#define US_PAR_MARK             0x00000600      /*!< \brief Marked parity. */
#define US_PAR_NO               0x00000800      /*!< \brief No parity. */
#define US_PAR_MULTIDROP        0x00000C00      /*!< \brief Multi-drop mode. */

#define US_NBSTOP               0x00003000      /*!< \brief Masks stop bit length. */
#define US_NBSTOP_1             0x00000000      /*!< \brief 1 stop bit. */
#define US_NBSTOP_1_5           0x00001000      /*!< \brief 1.5 stop bits. */
#define US_NBSTOP_2             0x00002000      /*!< \brief 2 stop bits. */

#define US_CHMODE                   0x0000C000  /*!< \brief Channel mode mask. */
#define US_CHMODE_NORMAL            0x00000000  /*!< \brief Normal mode. */
#define US_CHMODE_AUTOMATIC_ECHO    0x00004000  /*!< \brief Automatic echo. */
#define US_CHMODE_LOCAL_LOOPBACK    0x00008000  /*!< \brief Local loopback. */
#define US_CHMODE_REMOTE_LOOPBACK   0x0000C000  /*!< \brief Remote loopback. */

#define US_MODE9                0x00020000      /*!< \brief 9 bit mode. */

#define US_CLKO                 0x00040000      /*!< \brief Baud rate output enable */
/*@}*/

/*! \name Status and Interrupt Register */
/*@{*/
#define US_CSR_OFF              0x00000014      /*!< \brief USART status register offset. */
#define US0_CSR (USART0_BASE + US_CSR_OFF)      /*!< \brief Channel 0 status register address. */
#define US1_CSR (USART1_BASE + US_CSR_OFF)      /*!< \brief Channel 1 status register address. */
#define DBGU_CSR (DBGU_BASE + US_CSR_OFF)       /*!< \brief DBGU status register address. */

#define US_IER_OFF              0x00000008      /*!< \brief USART interrupt enable register offset. */
#define US0_IER (USART0_BASE + US_IER_OFF)      /*!< \brief Channel 0 interrupt enable register address. */
#define US1_IER (USART1_BASE + US_IER_OFF)      /*!< \brief Channel 1 interrupt enable register address. */
#define DBGU_IER (DBGU_BASE + US_IER_OFF)       /*!< \brief DBGU interrupt enable register address. */

#define US_IDR_OFF              0x0000000C      /*!< \brief USART interrupt disable register offset. */
#define US0_IDR (USART0_BASE + US_IDR_OFF)      /*!< \brief Channel 0 interrupt disable register address. */
#define US1_IDR (USART1_BASE + US_IDR_OFF)      /*!< \brief Channel 1 interrupt disable register address. */
#define DBGU_IDR (DBGU_BASE + US_IDR_OFF)       /*!< \brief DBGU interrupt disable register address. */

#define US_IMR_OFF              0x00000010      /*!< \brief USART interrupt mask register offset. */
#define US0_IMR (USART0_BASE + US_IMR_OFF)      /*!< \brief Channel 0 interrupt mask register address. */
#define US1_IMR (USART1_BASE + US_IMR_OFF)      /*!< \brief Channel 1 interrupt mask register address. */
#define DBGU_IMR (DBGU_BASE + US_IMR_OFF)       /*!< \brief DBGU interrupt mask register address. */

#define US_RXRDY                0x00000001      /*!< \brief Receiver ready */
#define US_TXRDY                0x00000002      /*!< \brief Transmitter ready */
#define US_RXBRK                0x00000004      /*!< \brief Receiver break */
#define US_ENDRX                0x00000008      /*!< \brief End of receiver PDC transfer */
#define US_ENDTX                0x00000010      /*!< \brief End of transmitter PDC transfer */
#define US_OVRE                 0x00000020      /*!< \brief Overrun error */
#define US_FRAME                0x00000040      /*!< \brief Framing error */
#define US_PARE                 0x00000080      /*!< \brief Parity error */
#define US_TIMEOUT              0x00000100      /*!< \brief Receiver timeout */
#define US_TXEMPTY              0x00000200      /*!< \brief Transmitter empty */

/*! \brief Baud rate calculation helper macro.
 *
 * \deprecated Use NutGetCpuClock() and calculate the divider value locally.
 */
#define AT91_US_BAUD(baud) ((NUT_CPU_FREQ / (8 * (baud)) + 1) / 2)
/*@}*/

/*! \name Receiver Holding Register */
/*@{*/
#define US_RHR_OFF              0x00000018      /*!< \brief USART receiver holding register offset. */
#define US0_RHR (USART0_BASE + US_RHR_OFF)      /*!< \brief Channel 0 receiver holding register address. */
#define US1_RHR (USART1_BASE + US_RHR_OFF)      /*!< \brief Channel 1 receiver holding register address. */
#define DBGU_RHR (DBGU_BASE + US_RHR_OFF)       /*!< \brief DBGU receiver holding register address. */
/*@}*/

/*! \name Transmitter Holding Register */
/*@{*/
#define US_THR_OFF              0x0000001C      /*!< \brief USART transmitter holding register offset. */
#define US0_THR (USART0_BASE + US_THR_OFF)      /*!< \brief Channel 0 transmitter holding register address. */
#define US1_THR (USART1_BASE + US_THR_OFF)      /*!< \brief Channel 1 transmitter holding register address. */
#define DBGU_THR (DBGU_BASE + US_THR_OFF)       /*!< \brief DBGU transmitter holding register address. */
/*@}*/

/*! \name Baud Rate Generator Register */
/*@{*/
#define US_BRGR_OFF             0x00000020      /*!< \brief USART baud rate register offset. */
#define US0_BRGR (USART0_BASE + US_BRGR_OFF)    /*!< \brief Channel 0 baud rate register address. */
#define US1_BRGR (USART1_BASE + US_BRGR_OFF)    /*!< \brief Channel 1 baud rate register address. */
#define DBGU_BRGR (DBGU_BASE + US_BRGR_OFF)     /*!< \brief DBGU baud rate register address. */
/*@}*/

/*! \name Receiver Timeout Register */
/*@{*/
#define US_RTOR_OFF             0x00000024      /*!< \brief USART receiver timeout register offset. */
#define US0_RTOR (USART0_BASE + US_RTOR_OFF)    /*!< \brief Channel 0 receiver timeout register address. */
#define US1_RTOR (USART1_BASE + US_RTOR_OFF)    /*!< \brief Channel 1 receiver timeout register address. */
/*@}*/

/*! \name Transmitter Time Guard Register */
/*@{*/
#define US_TTGR_OFF             0x00000028      /*!< \brief USART transmitter time guard register offset. */
#define US0_TTGR (USART0_BASE + US_TTGR_OFF)    /*!< \brief Channel 0 transmitter time guard register address. */
#define US1_TTGR (USART1_BASE + US_TTGR_OFF)    /*!< \brief Channel 1 transmitter time guard register address. */
/*@}*/

/*! \name Special Registers for DBGU */
/*@{*/
#define DBGU_CIDR (DBGU_BASE + 0x00000040)      /*!< \brief DBGU Chip ID Register. */
#define DBGU_EXID (DBGU_BASE + 0x00000044)      /*!< \brief DBGU Chip ID Extension Register. */
#define DBGU_FNR (DBGU_BASE + 0x00000048)       /*!< \brief DBGU Force NTRST Register. */
/*@}*/

/*@}*/

/*@} xgNutArchArmAt91Us */

/*!
 * \addtogroup xgNutArchArmAt91Emac
 */
/*@{*/

#define EMAC_BASE   0xFFFDC000          /*!< EMAC base address. */

#define EMAC_NCR    (EMAC_BASE + 0x00)    /*!< \brief Network Control Register. */
#define EMAC_LB         ((unsigned int) 0x1 <<  0) /*!< \brief  Loopback. Optional. When set, loopback signal is at high level. */
#define EMAC_LLB        ((unsigned int) 0x1 <<  1) /*!< \brief  Loopback local. */
#define EMAC_RE         ((unsigned int) 0x1 <<  2) /*!< \brief  Receive enable. */
#define EMAC_TE         ((unsigned int) 0x1 <<  3) /*!< \brief  Transmit enable. */
#define EMAC_MPE        ((unsigned int) 0x1 <<  4) /*!< \brief  Management port enable. */
#define EMAC_CLRSTAT    ((unsigned int) 0x1 <<  5) /*!< \brief  Clear statistics registers. */
#define EMAC_INCSTAT    ((unsigned int) 0x1 <<  6) /*!< \brief  Increment statistics registers. */
#define EMAC_WESTAT     ((unsigned int) 0x1 <<  7) /*!< \brief  Write enable for statistics registers. */
#define EMAC_BP         ((unsigned int) 0x1 <<  8) /*!< \brief  Back pressure. */
#define EMAC_TSTART     ((unsigned int) 0x1 <<  9) /*!< \brief  Start Transmission. */
#define EMAC_THALT      ((unsigned int) 0x1 << 10) /*!< \brief  Transmission Halt. */
#define EMAC_TPFR       ((unsigned int) 0x1 << 11) /*!< \brief  Transmit pause frame. */
#define EMAC_TZQ        ((unsigned int) 0x1 << 12) /*!< \brief  Transmit zero quantum pause frame. */

#define EMAC_NCFGR  (EMAC_BASE + 0x04)    /*!< \brief Network Configuration Register. */
#define EMAC_SPD        ((unsigned int) 0x1 <<  0) /*!< \brief  Speed. */
#define EMAC_FD         ((unsigned int) 0x1 <<  1) /*!< \brief  Full duplex. */
#define EMAC_JFRAME     ((unsigned int) 0x1 <<  3) /*!< \brief  Jumbo Frames. */
#define EMAC_CAF        ((unsigned int) 0x1 <<  4) /*!< \brief  Copy all frames. */
#define EMAC_NBC        ((unsigned int) 0x1 <<  5) /*!< \brief  No broadcast. */
#define EMAC_MTI        ((unsigned int) 0x1 <<  6) /*!< \brief  Multicast hash event enable. */
#define EMAC_UNI        ((unsigned int) 0x1 <<  7) /*!< \brief  Unicast hash enable. */
#define EMAC_BIG        ((unsigned int) 0x1 <<  8) /*!< \brief  Receive 1522 bytes. */
#define EMAC_EAE        ((unsigned int) 0x1 <<  9) /*!< \brief  External address match enable. */
#define EMAC_CLK        ((unsigned int) 0x3 << 10) /*!< \brief . */
#define EMAC_CLK_HCLK_8     ((unsigned int) 0x0 << 10) /*!< \brief  HCLK divided by 8. */
#define EMAC_CLK_HCLK_16    ((unsigned int) 0x1 << 10) /*!< \brief  HCLK divided by 16. */
#define EMAC_CLK_HCLK_32    ((unsigned int) 0x2 << 10) /*!< \brief  HCLK divided by 32. */
#define EMAC_CLK_HCLK_64    ((unsigned int) 0x3 << 10) /*!< \brief  HCLK divided by 64. */
#define EMAC_RTY        ((unsigned int) 0x1 << 12) /*!< \brief  . */
#define EMAC_PAE        ((unsigned int) 0x1 << 13) /*!< \brief  . */
#define EMAC_RBOF       ((unsigned int) 0x3 << 14) /*!< \brief  . */
#define EMAC_RBOF_OFFSET_0  ((unsigned int) 0x0 << 14) /*!< \brief  no offset from start of receive buffer. */
#define EMAC_RBOF_OFFSET_1  ((unsigned int) 0x1 << 14) /*!< \brief  one byte offset from start of receive buffer. */
#define EMAC_RBOF_OFFSET_2  ((unsigned int) 0x2 << 14) /*!< \brief  two bytes offset from start of receive buffer. */
#define EMAC_RBOF_OFFSET_3  ((unsigned int) 0x3 << 14) /*!< \brief  three bytes offset from start of receive buffer. */
#define EMAC_RLCE       ((unsigned int) 0x1 << 16) /*!< \brief  Receive Length field Checking Enable. */
#define EMAC_DRFCS      ((unsigned int) 0x1 << 17) /*!< \brief  Discard Receive FCS. */
#define EMAC_EFRHD      ((unsigned int) 0x1 << 18) /*!< \brief  . */
#define EMAC_IRXFCS     ((unsigned int) 0x1 << 19) /*!< \brief  Ignore RX FCS. */

#define EMAC_NSR    (EMAC_BASE + 0x08)    /*!< \brief Network Status Register. */
#define EMAC_LINKR      ((unsigned int) 0x1 <<  0) /*!< \brief  . */
#define EMAC_MDIO       ((unsigned int) 0x1 <<  1) /*!< \brief  . */
#define EMAC_IDLE       ((unsigned int) 0x1 <<  2) /*!< \brief  . */

#define EMAC_TSR    (EMAC_BASE + 0x14)    /*!< \brief Transmit Status Register. */
#define EMAC_UBR        ((unsigned int) 0x1 <<  0) /*!< \brief  . */
#define EMAC_COL        ((unsigned int) 0x1 <<  1) /*!< \brief  . */
#define EMAC_RLES       ((unsigned int) 0x1 <<  2) /*!< \brief  . */
#define EMAC_TGO        ((unsigned int) 0x1 <<  3) /*!< \brief  Transmit Go. */
#define EMAC_BEX        ((unsigned int) 0x1 <<  4) /*!< \brief  Buffers exhausted mid frame. */
#define EMAC_COMP       ((unsigned int) 0x1 <<  5) /*!< \brief  . */
#define EMAC_UND        ((unsigned int) 0x1 <<  6) /*!< \brief  . */

#define EMAC_RBQP   (EMAC_BASE + 0x18)    /*!< \brief Receive Buffer Queue Pointer. */

#define EMAC_TBQP   (EMAC_BASE + 0x1C)    /*!< \brief Transmit Buffer Queue Pointer. */

#define EMAC_RSR    (EMAC_BASE + 0x20)    /*!< \brief Receive Status Register. */
#define EMAC_BNA        ((unsigned int) 0x1 <<  0) /*!< \brief  . */
#define EMAC_REC        ((unsigned int) 0x1 <<  1) /*!< \brief  . */
#define EMAC_OVR        ((unsigned int) 0x1 <<  2) /*!< \brief  . */

#define EMAC_ISR    (EMAC_BASE + 0x24)    /*!< \brief Interrupt Status Register. */
#define EMAC_MFD        ((unsigned int) 0x1 <<  0) /*!< \brief  . */
#define EMAC_RCOMP      ((unsigned int) 0x1 <<  1) /*!< \brief  . */
#define EMAC_RXUBR      ((unsigned int) 0x1 <<  2) /*!< \brief  . */
#define EMAC_TXUBR      ((unsigned int) 0x1 <<  3) /*!< \brief  . */
#define EMAC_TUNDR      ((unsigned int) 0x1 <<  4) /*!< \brief  . */
#define EMAC_RLEX       ((unsigned int) 0x1 <<  5) /*!< \brief  . */
#define EMAC_TXERR      ((unsigned int) 0x1 <<  6) /*!< \brief  . */
#define EMAC_TCOMP      ((unsigned int) 0x1 <<  7) /*!< \brief  . */
#define EMAC_LINK       ((unsigned int) 0x1 <<  9) /*!< \brief  . */
#define EMAC_ROVR       ((unsigned int) 0x1 << 10) /*!< \brief  . */
#define EMAC_HRESP      ((unsigned int) 0x1 << 11) /*!< \brief  . */
#define EMAC_PFRE       ((unsigned int) 0x1 << 12) /*!< \brief  . */
#define EMAC_PTZ        ((unsigned int) 0x1 << 13) /*!< \brief  . */

#define EMAC_IER    (EMAC_BASE + 0x28)    /*!< \brief Interrupt Enable Register. */
#define EMAC_IDR    (EMAC_BASE + 0x2C)    /*!< \brief Interrupt Disable Register. */
#define EMAC_IMR    (EMAC_BASE + 0x30)    /*!< \brief Interrupt Mask Register. */
#define EMAC_MAN    (EMAC_BASE + 0x34)    /*!< \brief PHY Maintenance Register. */
#define EMAC_DATA       ((unsigned int) 0xFFFF <<  0) /*!< \brief  . */
#define EMAC_CODE       ((unsigned int) 0x3 << 16) /*!< \brief  . */
#define EMAC_REGA       ((unsigned int) 0x1F << 18) /*!< \brief  . */
#define EMAC_PHYA       ((unsigned int) 0x1F << 23) /*!< \brief  . */
#define EMAC_RW         ((unsigned int) 0x3 << 28) /*!< \brief  . */
#define EMAC_SOF        ((unsigned int) 0x3 << 30) /*!< \brief  . */

#define EMAC_PTR    (EMAC_BASE + 0x38)    /*!< \brief Pause Time Register. */
#define EMAC_PFR    (EMAC_BASE + 0x3C)    /*!< \brief Pause Frames received Register. */
#define EMAC_FTO    (EMAC_BASE + 0x40)    /*!< \brief Frames Transmitted OK Register. */
#define EMAC_SCF    (EMAC_BASE + 0x44)    /*!< \brief Single Collision Frame Register. */
#define EMAC_MCF    (EMAC_BASE + 0x48)    /*!< \brief Multiple Collision Frame Register. */
#define EMAC_FRO    (EMAC_BASE + 0x4C)    /*!< \brief Frames Received OK Register. */
#define EMAC_FCSE   (EMAC_BASE + 0x50)    /*!< \brief Frame Check Sequence Error Register. */
#define EMAC_ALE    (EMAC_BASE + 0x54)    /*!< \brief Alignment Error Register. */
#define EMAC_DTF    (EMAC_BASE + 0x58)    /*!< \brief Deferred Transmission Frame Register. */
#define EMAC_LCOL   (EMAC_BASE + 0x5C)    /*!< \brief Late Collision Register. */
#define EMAC_ECOL   (EMAC_BASE + 0x60)    /*!< \brief Excessive Collision Register. */
#define EMAC_TUND   (EMAC_BASE + 0x64)    /*!< \brief Transmit Underrun Error Register. */
#define EMAC_CSE    (EMAC_BASE + 0x68)    /*!< \brief Carrier Sense Error Register. */
#define EMAC_RRE    (EMAC_BASE + 0x6C)    /*!< \brief Receive Ressource Error Register. */
#define EMAC_ROV    (EMAC_BASE + 0x70)    /*!< \brief Receive Overrun Errors Register. */
#define EMAC_RSE    (EMAC_BASE + 0x74)    /*!< \brief Receive Symbol Errors Register. */
#define EMAC_ELE    (EMAC_BASE + 0x78)    /*!< \brief Excessive Length Errors Register. */
#define EMAC_RJA    (EMAC_BASE + 0x7C)    /*!< \brief Receive Jabbers Register. */
#define EMAC_USF    (EMAC_BASE + 0x80)    /*!< \brief Undersize Frames Register. */
#define EMAC_STE    (EMAC_BASE + 0x84)    /*!< \brief SQE Test Error Register. */
#define EMAC_RLE    (EMAC_BASE + 0x88)    /*!< \brief Receive Length Field Mismatch Register. */
#define EMAC_TPF    (EMAC_BASE + 0x8C)    /*!< \brief Transmitted Pause Frames Register. */
#define EMAC_HRB    (EMAC_BASE + 0x90)    /*!< \brief Hash Address Bottom[31:0]. */
#define EMAC_HRT    (EMAC_BASE + 0x94)    /*!< \brief Hash Address Top[63:32]. */
#define EMAC_SA1L   (EMAC_BASE + 0x98)    /*!< \brief Specific Address 1 Bottom, First 4 bytes. */
#define EMAC_SA1H   (EMAC_BASE + 0x9C)    /*!< \brief Specific Address 1 Top, Last 2 bytes. */
#define EMAC_SA2L   (EMAC_BASE + 0xA0)    /*!< \brief Specific Address 2 Bottom, First 4 bytes. */
#define EMAC_SA2H   (EMAC_BASE + 0xA4)    /*!< \brief Specific Address 2 Top, Last 2 bytes. */
#define EMAC_SA3L   (EMAC_BASE + 0xA8)    /*!< \brief Specific Address 3 Bottom, First 4 bytes. */
#define EMAC_SA3H   (EMAC_BASE + 0xAC)    /*!< \brief Specific Address 3 Top, Last 2 bytes. */
#define EMAC_SA4L   (EMAC_BASE + 0xB0)    /*!< \brief Specific Address 4 Bottom, First 4 bytes. */
#define EMAC_SA4H   (EMAC_BASE + 0xB4)    /*!< \brief Specific Address 4 Top, Last 2 bytes. */
#define EMAC_TID    (EMAC_BASE + 0xB8)    /*!< \brief Type ID Checking Register. */
#define EMAC_TPQ    (EMAC_BASE + 0xBC)    /*!< \brief Transmit Pause Quantum Register. */

#define EMAC_USRIO  (EMAC_BASE + 0xC0)    /*!< \brief USER Input/Output Register. */
#define EMAC_RMII       ((unsigned int) 0x1 <<  0) /*!< \brief  Reduce MII. */

#define EMAC_WOL    (EMAC_BASE + 0xC4)    /*!< \brief Wake On LAN Register. */
#define EMAC_IP         ((unsigned int) 0xFFFF <<  0) /*!< \brief  ARP request IP address. */
#define EMAC_MAG        ((unsigned int) 0x1 << 16) /*!< \brief  Magic packet event enable. */
#define EMAC_ARP        ((unsigned int) 0x1 << 17) /*!< \brief  ARP request event enable. */
#define EMAC_SA1        ((unsigned int) 0x1 << 18) /*!< \brief  Specific address register 1 event enable. */

#define EMAC_REV    (EMAC_BASE + 0xFC)    /*!< \brief Revision Register. */
#define EMAC_REVREF     ((unsigned int) 0xFFFF <<  0) /*!< \brief  . */
#define EMAC_PARTREF    ((unsigned int) 0xFFFF << 16) /*!< \brief  . */

/*@}*/

/*@} xgNutArchArmAt91Emac */

/*!
 * \addtogroup xgNutArchArmAt91Aic
 */
/*@{*/
#define AIC_BASE    0xFFFFF000  /*!< AIC base address. */

/*! \name Interrupt Source Mode Registers */
/*@{*/
/*! \brief Source mode register array.
 */
#define AIC_SMR(i)  (AIC_BASE + i * 4)

/*! \brief Priority mask.
 *
 * Priority levels can be between 0 (lowest) and 7 (highest).
 */
#define AIC_PRIOR                       0x00000007

/*! \brief Interrupt source type mask.
 *
 * Internal interrupts can level sensitive or edge triggered.
 *
 * External interrupts can triggered on positive or negative levels or
 * on rising or falling edges.
 */
#define AIC_SRCTYPE                     0x00000060

#define AIC_SRCTYPE_INT_LEVEL_SENSITIVE 0x00000000      /*!< \brief Internal level sensitive. */
#define AIC_SRCTYPE_INT_EDGE_TRIGGERED  0x00000020      /*!< \brief Internal edge triggered. */
#define AIC_SRCTYPE_EXT_LOW_LEVEL       0x00000000      /*!< \brief External low level. */
#define AIC_SRCTYPE_EXT_NEGATIVE_EDGE   0x00000020      /*!< \brief External falling edge. */
#define AIC_SRCTYPE_EXT_HIGH_LEVEL      0x00000040      /*!< \brief External high level. */
#define AIC_SRCTYPE_EXT_POSITIVE_EDGE   0x00000060      /*!< \brief External rising edge. */
/*@}*/

/*! \name Interrupt Source Vector Registers */
/*@{*/
/*! \brief Source vector register array.
 *
 * Stores the addresses of the corresponding interrupt handlers.
 */
#define AIC_SVR(i)  (AIC_BASE + 0x80 + i * 4)
/*@}*/

/*! \name Interrupt Vector Register */
/*@{*/
#define AIC_IVR     (AIC_BASE + 0x100)  /*!< \brief IRQ vector register address. */
/*@}*/

/*! \name Fast Interrupt Vector Register */
/*@{*/
#define AIC_FVR     (AIC_BASE + 0x104)  /*!< \brief FIQ vector register address. */
/*@}*/

/*! \name Interrupt Status Register */
/*@{*/
#define AIC_ISR     (AIC_BASE + 0x108)  /*!< \brief Interrupt status register address. */
#define AIC_IRQID               0x0000001F      /*!< \brief Current interrupt identifier mask. */
/*@}*/

/*! \name Interrupt Pending Register */
/*@{*/
#define AIC_IPR     (AIC_BASE + 0x10C)  /*!< \brief Interrupt pending register address. */
/*@}*/

/*! \name Interrupt Mask Register */
/*@{*/
#define AIC_IMR     (AIC_BASE + 0x110)  /*!< \brief Interrupt mask register address. */
/*@}*/

/*! \name Interrupt Core Status Register */
/*@{*/
#define AIC_CISR    (AIC_BASE + 0x114)  /*!< \brief Core interrupt status register address. */
#define AIC_NFIQ                0x00000001      /*!< \brief Core FIQ Status */
#define AIC_NIRQ                0x00000002      /*!< \brief Core IRQ Status */
/*@}*/

/*! \name Interrupt Enable Command Register */
/*@{*/
#define AIC_IECR    (AIC_BASE + 0x120)  /*!< \brief Interrupt enable command register address. */
/*@}*/

/*! \name Interrupt Disable Command Register */
/*@{*/
#define AIC_IDCR    (AIC_BASE + 0x124)  /*!< \brief Interrupt disable command register address. */
/*@}*/

/*! \name Interrupt Clear Command Register */
/*@{*/
#define AIC_ICCR    (AIC_BASE + 0x128)  /*!< \brief Interrupt clear command register address. */
/*@}*/

/*! \name Interrupt Set Command Register */
/*@{*/
#define AIC_ISCR    (AIC_BASE + 0x12C)  /*!< \brief Interrupt set command register address. */
/*@}*/

/*! \name End Of Interrupt Command Register */
/*@{*/
#define AIC_EOICR   (AIC_BASE + 0x130)  /*!< \brief End of interrupt command register address. */
/*@}*/

/*! \name Spurious Interrupt Vector Register */
/*@{*/
#define AIC_SPU     (AIC_BASE + 0x134)  /*!< \brief Spurious vector register address. */
/*@}*/

/*!
 * \addtogroup xgNutArchArmAt91Pio
 */
/*@{*/
#define PIOA_BASE    0xFFFFF400          /*!< \brief PIO base address. */
#define PIOA_PER     (PIOA_BASE + 0x00)   /*!< \brief PIO enable register. */
#define PIOA_PDR     (PIOA_BASE + 0x04)   /*!< \brief PIO disable register. */
#define PIOA_PSR     (PIOA_BASE + 0x08)   /*!< \brief PIO status register. */
#define PIOA_OER     (PIOA_BASE + 0x10)   /*!< \brief Output enable register. */
#define PIOA_ODR     (PIOA_BASE + 0x14)   /*!< \brief Output disable register. */
#define PIOA_OSR     (PIOA_BASE + 0x18)   /*!< \brief Output status register. */
#define PIOA_IFER    (PIOA_BASE + 0x20)   /*!< \brief Input filter enable register. */
#define PIOA_IFDR    (PIOA_BASE + 0x24)   /*!< \brief Input filter disable register. */
#define PIOA_IFSR    (PIOA_BASE + 0x28)   /*!< \brief Input filter status register. */
#define PIOA_SODR    (PIOA_BASE + 0x30)   /*!< \brief Set output data register. */
#define PIOA_CODR    (PIOA_BASE + 0x34)   /*!< \brief Clear output data register. */
#define PIOA_ODSR    (PIOA_BASE + 0x38)   /*!< \brief Output data status register. */
#define PIOA_PDSR    (PIOA_BASE + 0x3C)   /*!< \brief Pin data status register. */
#define PIOA_IER     (PIOA_BASE + 0x40)   /*!< \brief Interrupt enable register. */
#define PIOA_IDR     (PIOA_BASE + 0x44)   /*!< \brief Interrupt disable register. */
#define PIOA_IMR     (PIOA_BASE + 0x48)   /*!< \brief Interrupt mask register. */
#define PIOA_ISR     (PIOA_BASE + 0x4C)   /*!< \brief Interrupt status register. */

#define PIOB_BASE    0xFFFFF600          /*!< \brief PIO base address. */
#define PIOB_PER     (PIOB_BASE + 0x00)   /*!< \brief PIO enable register. */
#define PIOB_PDR     (PIOB_BASE + 0x04)   /*!< \brief PIO disable register. */
#define PIOB_PSR     (PIOB_BASE + 0x08)   /*!< \brief PIO status register. */
#define PIOB_OER     (PIOB_BASE + 0x10)   /*!< \brief Output enable register. */
#define PIOB_ODR     (PIOB_BASE + 0x14)   /*!< \brief Output disable register. */
#define PIOB_OSR     (PIOB_BASE + 0x18)   /*!< \brief Output status register. */
#define PIOB_IFER    (PIOB_BASE + 0x20)   /*!< \brief Input filter enable register. */
#define PIOB_IFDR    (PIOB_BASE + 0x24)   /*!< \brief Input filter disable register. */
#define PIOB_IFSR    (PIOB_BASE + 0x28)   /*!< \brief Input filter status register. */
#define PIOB_SODR    (PIOB_BASE + 0x30)   /*!< \brief Set output data register. */
#define PIOB_CODR    (PIOB_BASE + 0x34)   /*!< \brief Clear output data register. */
#define PIOB_ODSR    (PIOB_BASE + 0x38)   /*!< \brief Output data status register. */
#define PIOB_PDSR    (PIOB_BASE + 0x3C)   /*!< \brief Pin data status register. */
#define PIOB_IER     (PIOB_BASE + 0x40)   /*!< \brief Interrupt enable register. */
#define PIOB_IDR     (PIOB_BASE + 0x44)   /*!< \brief Interrupt disable register. */
#define PIOB_IMR     (PIOB_BASE + 0x48)   /*!< \brief Interrupt mask register. */
#define PIOB_ISR     (PIOB_BASE + 0x4C)   /*!< \brief Interrupt status register. */
/*@} xgNutArchArmAt91Pio */

/*!
 * \addtogroup xgNutArchArmAt91Pmc
 */
/*@{*/
#define PMC_BASE    0xFFFFFC00  /*!< \brief PMC base address. */

#define PMC_SCER    (PMC_BASE + 0x00)   /*!< \brief System Clock Enable Register. */
#define PMC_PCK         ((unsigned int) 0x1 <<  0) /*!< \brief Processor Clock. */
#define PMC_UDP         ((unsigned int) 0x1 <<  7) /*!< \brief USB Device Port Clock. */
#define PMC_PCK0        ((unsigned int) 0x1 <<  8) /*!< \brief Programmable Clock Output. */
#define PMC_PCK1        ((unsigned int) 0x1 <<  9) /*!< \brief Programmable Clock Output. */
#define PMC_PCK2        ((unsigned int) 0x1 << 10) /*!< \brief Programmable Clock Output. */

#define PMC_SCDR    (PMC_BASE + 0x04)   /*!< \brief System Clock Disable Register. */
#define PMC_SCSR    (PMC_BASE + 0x08)   /*!< \brief System Clock Status Register. */
#define PMC_PCER    (PMC_BASE + 0x10)   /*!< \brief Peripheral Clock Enable Register. */
#define PMC_PCDR    (PMC_BASE + 0x14)   /*!< \brief Peripheral Clock Disable Register. */
#define PMC_PCSR    (PMC_BASE + 0x18)   /*!< \brief Peripheral Clock Status Register. */

#define CKGR_MOR        (PMC_BASE + 0x20)   /*!< \brief Main Oscillator Register. */
#define CKGR_MOSCEN     ((unsigned int) 0x1 <<  0) /*!< \brief Main Oscillator Enable. */
#define CKGR_OSCBYPASS  ((unsigned int) 0x1 <<  1) /*!< \brief Main Oscillator Bypass. */
#define CKGR_OSCOUNT    ((unsigned int) 0xFF <<  8) /*!< \brief Main Oscillator Start-up Time. */

#define CKGR_MCFR       (PMC_BASE + 0x24)   /*!< \brief Main Clock  Frequency Register. */
#define CKGR_MAINF      ((unsigned int) 0xFFFF <<  0) /*!< \brief Main Clock Frequency. */
#define CKGR_MAINRDY    ((unsigned int) 0x1 << 16) /*!< \brief Main Clock Ready. */

#define CKGR_PLLR       (PMC_BASE + 0x2C)   /*!< \brief PLL Register. */
#define CKGR_DIV        ((unsigned int) 0xFF <<  0) /*!< \brief Divider Selected. */
#define CKGR_DIV_0      ((unsigned int) 0x0) /*!< \brief Divider output is 0. */
#define CKGR_DIV_BYPASS ((unsigned int) 0x1) /*!< \brief Divider is bypassed. */
#define CKGR_PLLCOUNT   ((unsigned int) 0x3F <<  8) /*!< \brief PLL Counter. */
#define CKGR_OUT        ((unsigned int) 0x3 << 14) /*!< \brief PLL Output Frequency Range. */
#define CKGR_OUT_0      ((unsigned int) 0x0 << 14) /*!< \brief Please refer to the PLL datasheet. */
#define CKGR_OUT_1      ((unsigned int) 0x1 << 14) /*!< \brief Please refer to the PLL datasheet. */
#define CKGR_OUT_2      ((unsigned int) 0x2 << 14) /*!< \brief Please refer to the PLL datasheet. */
#define CKGR_OUT_3      ((unsigned int) 0x3 << 14) /*!< \brief Please refer to the PLL datasheet. */
#define CKGR_MUL        ((unsigned int) 0x7FF << 16) /*!< \brief PLL Multiplier. */
#define CKGR_USBDIV     ((unsigned int) 0x3 << 28) /*!< \brief Divider for USB Clocks. */
#define CKGR_USBDIV_0   ((unsigned int) 0x0 << 28) /*!< \brief Divider output is PLL clock output. */
#define CKGR_USBDIV_1   ((unsigned int) 0x1 << 28) /*!< \brief Divider output is PLL clock output divided by 2. */
#define CKGR_USBDIV_2   ((unsigned int) 0x2 << 28) /*!< \brief Divider output is PLL clock output divided by 4. */

#define PMC_MCKR    (PMC_BASE + 0x30)   /*!< \brief Master Clock Register. */
#define PMC_CSS         ((unsigned int) 0x3 <<  0) /*!< \brief Programmable Clock Selection. */
#define PMC_CSS_SLOW_CLK ((unsigned int) 0x0) /*!< \brief Slow Clock is selected. */
#define PMC_CSS_MAIN_CLK ((unsigned int) 0x1) /*!< \brief Main Clock is selected. */
#define PMC_CSS_PLL_CLK ((unsigned int) 0x3) /*!< \brief Clock from PLL is selected. */
#define PMC_PRES        ((unsigned int) 0x7 <<  2) /*!< \brief Programmable Clock Prescaler. */
#define PMC_PRES_CLK    ((unsigned int) 0x0 <<  2) /*!< \brief Selected clock. */
#define PMC_PRES_CLK_2  ((unsigned int) 0x1 <<  2) /*!< \brief Selected clock divided by 2. */
#define PMC_PRES_CLK_4  ((unsigned int) 0x2 <<  2) /*!< \brief Selected clock divided by 4. */
#define PMC_PRES_CLK_8  ((unsigned int) 0x3 <<  2) /*!< \brief Selected clock divided by 8. */
#define PMC_PRES_CLK_16 ((unsigned int) 0x4 <<  2) /*!< \brief Selected clock divided by 16. */
#define PMC_PRES_CLK_32 ((unsigned int) 0x5 <<  2) /*!< \brief Selected clock divided by 32. */
#define PMC_PRES_CLK_64 ((unsigned int) 0x6 <<  2) /*!< \brief Selected clock divided by 64. */

#define PMC_PCKR0   (PMC_BASE + 0x38)   /*!< \brief Programmable Clock Register. */
#define PMC_PCKR1   (PMC_BASE + 0x3C)   /*!< \brief Programmable Clock Register. */
#define PMC_PCKR2   (PMC_BASE + 0x40)   /*!< \brief Programmable Clock Register. */
#define PMC_PCKR3   (PMC_BASE + 0x44)   /*!< \brief Programmable Clock Register. */

#define PMC_IER     (PMC_BASE + 0x60)   /*!< \brief Interrupt Enable Register. */
#define PMC_IDR     (PMC_BASE + 0x64)   /*!< \brief Interrupt Disable Register. */
#define PMC_SR      (PMC_BASE + 0x68)   /*!< \brief Status Register. */
#define PMC_IMR     (PMC_BASE + 0x6C)   /*!< \brief Interrupt Mask Register. */
#define PMC_MOSCS       ((unsigned int) 0x1 <<  0) /*!< \brief MOSC Status/Enable/Disable/Mask. */
#define PMC_LOCK        ((unsigned int) 0x1 <<  2) /*!< \brief PLL Status/Enable/Disable/Mask. */
#define PMC_MCKRDY      ((unsigned int) 0x1 <<  3) /*!< \brief MCK_RDY Status/Enable/Disable/Mask. */
#define PMC_PCK0RDY     ((unsigned int) 0x1 <<  8) /*!< \brief PCK0_RDY Status/Enable/Disable/Mask. */
#define PMC_PCK1RDY     ((unsigned int) 0x1 <<  9) /*!< \brief PCK1_RDY Status/Enable/Disable/Mask. */
#define PMC_PCK2RDY     ((unsigned int) 0x1 << 10) /*!< \brief PCK2_RDY Status/Enable/Disable/Mask. */

/*@} xgNutArchArmAt91Pmc */

/*!
 * \addtogroup xgNutArchArmAt91Rstc
 */
/*@{*/
#define RSTC_BASE       0xFFFFFD00          /*!< \brief Resect controller register base address. */

#define RSTC_RCR        (RSTC_BASE + 0x00) /*!< \brief Reset Control Register. */
#define RSTC_PROCRST    ((unsigned int) 0x1 <<  0) /*!< \brief Processor Reset. */
#define RSTC_PERRST     ((unsigned int) 0x1 <<  2) /*!< \brief Peripheral Reset. */
#define RSTC_EXTRST     ((unsigned int) 0x1 <<  3) /*!< \brief External Reset. */
#define RSTC_KEY        ((unsigned int) 0xFF << 24) /*!< \brief Password. */

#define RSTC_RMR        (RSTC_BASE + 0x08) /*!< \brief Reset Mode Register. */
#define RSTC_URSTEN     ((unsigned int) 0x1 <<  0) /*!< \brief User Reset Enable. */
#define RSTC_URSTIEN    ((unsigned int) 0x1 <<  4) /*!< \brief User Reset Interrupt Enable. */
#define RSTC_ERSTL      ((unsigned int) 0xF <<  8) /*!< \brief User Reset Length. */
#define RSTC_BODIEN     ((unsigned int) 0x1 << 16) /*!< \brief Brownout Detection Interrupt Enable. */

#define RSTC_RSR        (RSTC_BASE + 0x04) /*!< \brief Reset Status Register. */
#define RSTC_URSTS      ((unsigned int) 0x1 <<  0) /*!< \brief User Reset Status. */
#define RSTC_BODSTS     ((unsigned int) 0x1 <<  1) /*!< \brief Brownout Detection Status. */
#define RSTC_RSTTYP     ((unsigned int) 0x7 <<  8) /*!< \brief Reset Type. */
#define RSTC_RSTTYP_POWERUP     ((unsigned int) 0x0 <<  8) /*!< \brief Power-up Reset. VDDCORE rising.. */
#define RSTC_RSTTYP_WAKEUP      ((unsigned int) 0x1 <<  8) /*!< \brief WakeUp Reset. VDDCORE rising.. */
#define RSTC_RSTTYP_WATCHDOG    ((unsigned int) 0x2 <<  8) /*!< \brief Watchdog Reset. Watchdog overflow occured.. */
#define RSTC_RSTTYP_SOFTWARE    ((unsigned int) 0x3 <<  8) /*!< \brief Software Reset. Processor reset required by the software.. */
#define RSTC_RSTTYP_USER        ((unsigned int) 0x4 <<  8) /*!< \brief User Reset. NRST pin detected low.. */
#define RSTC_RSTTYP_BROWNOUT    ((unsigned int) 0x5 <<  8) /*!< \brief Brownout Reset occured.. */
#define RSTC_NRSTL      ((unsigned int) 0x1 << 16) /*!< \brief NRST pin level. */
#define RSTC_SRCMP      ((unsigned int) 0x1 << 17) /*!< \brief Software Reset Command in Progress.. */

/*@} xgNutArchArmAt91Rstc */

/*!
 * \addtogroup xgNutArchArmAt91Wd
 */
/*@{*/
#define WDT_BASE     0xFFFFFD40  /*!< \brief Watch Dog register base address. */

#define WDT_WDCR        (WDT_BASE + 0x00) /* Watchdog Control Register. */
#define WDT_WDRSTT     ((unsigned int) 0x1 <<  0) /* Watchdog Restart. */
#define WDT_KEY        ((unsigned int) 0xFF << 24) /* Watchdog KEY Password. */

#define WDT_WDSR        (WDT_BASE + 0x04) /* Watchdog Status Register. */
#define WDT_WDUNF      ((unsigned int) 0x1 <<  0) /* Watchdog Underflow. */
#define WDT_WDERR      ((unsigned int) 0x1 <<  1) /* Watchdog Error. */

#define WDT_WDMR        (WDT_BASE + 0x08) /* Watchdog Mode Register. */
#define WDT_WDV        ((unsigned int) 0xFFF <<  0) /* Watchdog Timer Restart. */
#define WDT_WDFIEN     ((unsigned int) 0x1 << 12) /* Watchdog Fault Interrupt Enable. */
#define WDT_WDRSTEN    ((unsigned int) 0x1 << 13) /* Watchdog Reset Enable. */
#define WDT_WDRPROC    ((unsigned int) 0x1 << 14) /* Watchdog Timer Restart. */
#define WDT_WDDIS      ((unsigned int) 0x1 << 15) /* Watchdog Disable. */
#define WDT_WDD        ((unsigned int) 0xFFF << 16) /* Watchdog Delta Value. */
#define WDT_WDDBGHLT   ((unsigned int) 0x1 << 28) /* Watchdog Debug Halt. */
#define WDT_WDIDLEHLT  ((unsigned int) 0x1 << 29) /* Watchdog Idle Halt. */

/*@} xgNutArchArmAt91Wd */

#define MC_BASE     0xFFFFFF00          /* Memory controller base. */

#define MC_RCR      (MC_BASE + 0x00)    /* MC Remap Control Register. */
#define MC_RCB          ((unsigned int) 0x1 <<  0) /* Remap Command Bit. */

#define MC_ASR      (MC_BASE + 0x04)    /* MC Abort Status Register. */
#define MC_UNDADD       ((unsigned int) 0x1 <<  0) /* Undefined Addess Abort Status. */
#define MC_MISADD       ((unsigned int) 0x1 <<  1) /* Misaligned Addess Abort Status. */
#define MC_ABTSZ        ((unsigned int) 0x3 <<  8) /* Abort Size Status. */
#define MC_ABTSZ_BYTE   ((unsigned int) 0x0 <<  8) /* Byte. */
#define MC_ABTSZ_HWORD  ((unsigned int) 0x1 <<  8) /* Half-word. */
#define MC_ABTSZ_WORD   ((unsigned int) 0x2 <<  8) /* Word. */
#define MC_ABTTYP       ((unsigned int) 0x3 << 10) /* Abort Type Status. */
#define MC_ABTTYP_DATAR ((unsigned int) 0x0 << 10) /* Data Read. */
#define MC_ABTTYP_DATAW ((unsigned int) 0x1 << 10) /* Data Write. */
#define MC_ABTTYP_FETCH ((unsigned int) 0x2 << 10) /* Code Fetch. */
#define MC_MST0         ((unsigned int) 0x1 << 16) /* Master 0 Abort Source. */
#define MC_MST1         ((unsigned int) 0x1 << 17) /* Master 1 Abort Source. */
#define MC_SVMST0       ((unsigned int) 0x1 << 24) /* Saved Master 0 Abort Source. */
#define MC_SVMST1       ((unsigned int) 0x1 << 25) /* Saved Master 1 Abort Source. */

#define MC_AASR     (MC_BASE + 0x08)    /* MC Abort Address Status Register. */

#define MC_FMR      (MC_BASE + 0x60)    /* MC Flash Mode Register. */
#define MC_FRDY         ((unsigned int) 0x1 <<  0) /* Flash Ready. */
#define MC_LOCKE        ((unsigned int) 0x1 <<  2) /* Lock Error. */
#define MC_PROGE        ((unsigned int) 0x1 <<  3) /* Programming Error. */
#define MC_NEBP         ((unsigned int) 0x1 <<  7) /* No Erase Before Programming. */
#define MC_FWS          ((unsigned int) 0x3 <<  8) /* Flash Wait State. */
#define MC_FWS_0FWS     ((unsigned int) 0x0 <<  8) /* 1 cycle for Read, 2 for Write operations. */
#define MC_FWS_1FWS     ((unsigned int) 0x1 <<  8) /* 2 cycles for Read, 3 for Write operations. */
#define MC_FWS_2FWS     ((unsigned int) 0x2 <<  8) /* 3 cycles for Read, 4 for Write operations. */
#define MC_FWS_3FWS     ((unsigned int) 0x3 <<  8) /* 4 cycles for Read, 4 for Write operations. */
#define MC_FMCN         ((unsigned int) 0xFF << 16)/* Flash Microsecond Cycle Number. */

#define MC_FCR      (MC_BASE + 0x64)    /* MC Flash Command Register. */
#define MC_FCMD         ((unsigned int) 0xF <<  0) /* Flash Command. */
#define MC_FCMD_START_PROG ((unsigned int) 0x1) /* Starts the programming of th epage specified by PAGEN.. */
#define MC_FCMD_LOCK       ((unsigned int) 0x2) /* Starts a lock sequence of the sector defined by the bits 4 to 7 of the field PAGEN.. */
#define MC_FCMD_PROG_AND_LOCK ((unsigned int) 0x3) /* The lock sequence automatically happens after the programming sequence is completed.. */
#define MC_FCMD_UNLOCK     ((unsigned int) 0x4) /* Starts an unlock sequence of the sector defined by the bits 4 to 7 of the field PAGEN.. */
#define MC_FCMD_ERASE_ALL  ((unsigned int) 0x8) /* Starts the erase of the entire flash.If at least a page is locked, the command is cancelled.. */
#define MC_FCMD_SET_GP_NVM ((unsigned int) 0xB) /* Set General Purpose NVM bits.. */
#define MC_FCMD_CLR_GP_NVM ((unsigned int) 0xD) /* Clear General Purpose NVM bits.. */
#define MC_FCMD_SET_SECURITY ((unsigned int) 0xF) /* Set Security Bit.. */
#define MC_PAGEN        ((unsigned int) 0x3FF <<  8) /* Page Number. */
#define MC_KEY          ((unsigned int) 0xFF << 24) /* Writing Protect Key. */

#define MC_FSR      (MC_BASE + 0x68)    /* MC Flash Status Register. */
#define MC_SECURITY     ((unsigned int) 0x1 <<  4) /* Security Bit Status. */
#define MC_GPNVM0       ((unsigned int) 0x1 <<  8) /* Sector 0 Lock Status. */
#define MC_GPNVM1       ((unsigned int) 0x1 <<  9) /* Sector 1 Lock Status. */
#define MC_GPNVM2       ((unsigned int) 0x1 << 10) /* Sector 2 Lock Status. */
#define MC_GPNVM3       ((unsigned int) 0x1 << 11) /* Sector 3 Lock Status. */
#define MC_GPNVM4       ((unsigned int) 0x1 << 12) /* Sector 4 Lock Status. */
#define MC_GPNVM5       ((unsigned int) 0x1 << 13) /* Sector 5 Lock Status. */
#define MC_GPNVM6       ((unsigned int) 0x1 << 14) /* Sector 6 Lock Status. */
#define MC_GPNVM7       ((unsigned int) 0x1 << 15) /* Sector 7 Lock Status. */
#define MC_LOCKS0       ((unsigned int) 0x1 << 16) /* Sector 0 Lock Status. */
#define MC_LOCKS1       ((unsigned int) 0x1 << 17) /* Sector 1 Lock Status. */
#define MC_LOCKS2       ((unsigned int) 0x1 << 18) /* Sector 2 Lock Status. */
#define MC_LOCKS3       ((unsigned int) 0x1 << 19) /* Sector 3 Lock Status. */
#define MC_LOCKS4       ((unsigned int) 0x1 << 20) /* Sector 4 Lock Status. */
#define MC_LOCKS5       ((unsigned int) 0x1 << 21) /* Sector 5 Lock Status. */
#define MC_LOCKS6       ((unsigned int) 0x1 << 22) /* Sector 6 Lock Status. */
#define MC_LOCKS7       ((unsigned int) 0x1 << 23) /* Sector 7 Lock Status. */
#define MC_LOCKS8       ((unsigned int) 0x1 << 24) /* Sector 8 Lock Status. */
#define MC_LOCKS9       ((unsigned int) 0x1 << 25) /* Sector 9 Lock Status. */
#define MC_LOCKS10      ((unsigned int) 0x1 << 26) /* Sector 10 Lock Status. */
#define MC_LOCKS11      ((unsigned int) 0x1 << 27) /* Sector 11 Lock Status. */
#define MC_LOCKS12      ((unsigned int) 0x1 << 28) /* Sector 12 Lock Status. */
#define MC_LOCKS13      ((unsigned int) 0x1 << 29) /* Sector 13 Lock Status. */
#define MC_LOCKS14      ((unsigned int) 0x1 << 30) /* Sector 14 Lock Status. */
#define MC_LOCKS15      ((unsigned int) 0x1 << 31) /* Sector 15 Lock Status. */


#ifdef __GNUC__

/*!
 * \brief Interrupt entry.
 */
#define IRQ_ENTRY() \
    asm volatile("sub   lr, lr,#4"          "\n\t"  /* Adjust LR */ \
                 "stmfd sp!,{r0-r12,lr}"    "\n\t"  /* Save registers on IRQ stack. */ \
                 "mrs   r1, spsr"           "\n\t"  /* Save SPSR */ \
                 "stmfd sp!,{r1}"           "\n\t")     /* */

/*!
 * \brief Interrupt exit.
 */
#define IRQ_EXIT() \
    asm volatile("ldmfd sp!, {r1}"          "\n\t"  /* Restore SPSR */ \
                 "msr   spsr_c, r1"         "\n\t"  /* */ \
                 "ldr   r0, =0xFFFFF000"    "\n\t"  /* End of interrupt. */ \
                 "str   r0, [r0, #0x130]"   "\n\t"  /* */ \
                 "ldmfd sp!, {r0-r12, pc}^" "\n\t")     /* Restore registers and return. */

/*!
 * \brief Fast interrupt entry.
 */
#define FIQ_ENTRY() \
    asm volatile("sub   lr, lr,#4"          "\n\t"  /* Adjust LR */ \
                 "stmfd sp!,{r0-r7,lr}"    "\n\t"  /* Save registers on IRQ stack. */ \
                 "mrs   r1, spsr"           "\n\t"  /* Save SPSR */ \
                 "stmfd sp!,{r1}"           "\n\t")     /* */

/*!
 * \brief Fast interrupt exit.
 */
#define FIQ_EXIT() \
    asm volatile("ldmfd sp!, {r1}"          "\n\t"  /* Restore SPSR */ \
                 "msr   spsr_c, r1"         "\n\t"  /* */ \
                 "ldr   r0, =0xFFFFF000"    "\n\t"  /* End of interrupt. */ \
                 "str   r0, [r0, #0x130]"   "\n\t"  /* */ \
                 "ldmfd sp!, {r0-r7, pc}^" "\n\t")     /* Restore registers and return. */

#else /* __IMAGECRAFT__ */

#define IRQ_ENTRY() \
    asm("sub   lr, lr,#4\n" \
        "stmfd sp!,{r0-r12,lr}\n" \
        "mrs   r1, spsr\n" \
        "stmfd sp!,{r1}\n")

#define IRQ_EXIT() \
    asm("ldmfd sp!, {r1}\n" \
        "msr   spsr_c, r1\n" \
        ";ldr   r0, =0xFFFFF000\n" /* ICCARM: FIXME! */ \
        "str   r0, [r0, #0x130]\n" \
        "ldmfd sp!, {r0-r12, pc}^")

#define FIQ_ENTRY() \
    asm("sub   lr, lr,#4\n" \
        "stmfd sp!,{r0-r7,lr}\n" \
        "mrs   r1, spsr\n" \
        "stmfd sp!,{r1}\n")

#define FIQ_EXIT() \
    asm("ldmfd sp!, {r1}\n" \
        "msr   spsr_c, r1\n" \
        ";ldr   r0, =0xFFFFF000\n" /* ICCARM: FIXME! */ \
        "str   r0, [r0, #0x130]\n" \
        "ldmfd sp!, {r0-r7, pc}^")

#endif

/*@} xgNutArchArmAt91Aic */

/*! \addtogroup xgNutArchArmAt91 */
/*@{*/

/*! \name Peripheral Identifiers and Interrupts */
/*@{*/
#define FIQ_ID      0           /*!< \brief Fast interrupt ID. */
#define SYSIRQ_ID   1           /*!< \brief System interrupt ID. */
#define PIOA_ID     2           /*!< \brief Parallel I/O controller ID. */
#define PIOB_ID     3           /*!< \brief Parallel I/O controller ID. */
#define SPI0_ID     4           /*!< \brief Serial peripheral interface 0 ID. */
#define SPI1_ID     5           /*!< \brief Serial peripheral interface 1 ID. */
#define US0_ID      6           /*!< \brief USART 0 ID. */
#define US1_ID      7           /*!< \brief USART 1 ID. */
#define TC0_ID      12          /*!< \brief Timer 0 ID. */
#define TC1_ID      13          /*!< \brief Timer 1 ID. */
#define TC2_ID      14          /*!< \brief Timer 2 ID. */
#define EMAC_ID     16          /*!< \brief Ethernet MAC. */
#define IRQ0_ID     30          /*!< \brief External interrupt 0 ID. */
#define IRQ1_ID     31          /*!< \brief External interrupt 1 ID. */
/*@}*/
/*@} xgNutArchArmAt91 */

#endif                          /* _ARCH_ARM_SAM7X_H_ */
